#ifndef BASIC_H
#define BASIC_H

#include <vector>
#include <string>
#include <list>
#include "media_file.h"
#include "algorithm.h"

using namespace std;

class MainFrame;

#define MAX_ROWS 3

class PixalcoApp : public wxApp
{
public:
    PixalcoApp();
    virtual ~PixalcoApp();
	virtual bool OnInit();
    
    void AddMediaFile(const string& displayName_, const string& fileName_);
    MediaFile * GetMediaFile(const string& displayName_);
    void RemoveMediaFile(const string& displayName_);
    void RemoveAllMediaFiles(bool updateMainFrame_=false);
    void SelectMediaFile(const string& displayName_);

    void RunMovie(const string& displayName_, const string& algorithmName_);
    void StopMovie(const string& displayName_);
    void StepMovie(const string& displayName_, const string& algorithmName_);
    void SeekMovie(const string& displayName_, long pos_);
    
    void AddAlgorithm(Algorithm *algorithm_);
    Algorithm* GetAlgorithm(const string& name_);
    void ExecuteAlgorithm(const string& name_);

    wxImage* GetImage(int row_, int age_); // age_ == 0 is current, otherwise from buffer

    // the movie has it's own buffer with images which were not yet 
    wxImage* GetImageFromMovieBuffer(int age_); 
    
protected:
    void AddImageObjToMovieBuffer(ImageObject * imgObj_);
    void AddImageObjToBuffer(ImageObject * imgObj_, int line_, int row_);
    ImageObject* GetImageObjFromBuffer(int line_, int row_, int age_);
    void SetMediaFile(MediaFile * info_);
    
private:
    void RemoveAlgorithm(const string& name_);
    void RemoveAllAlgorithms();
        
    MainFrame *         mMainFrame;
    vector<MediaFile>   mMediaFiles;
    vector<Algorithm*>  mAlgorithms;
    list<ImageObject*>  *mImageObjBuffer[MAX_ROWS]; // TODO: needs to support each line and row
    list<ImageObject*>  mMovieBuffer;
};

DECLARE_APP(PixalcoApp);
#define PIXALCO static_cast<PixalcoApp&>(wxGetApp())

#endif

